
//******************************************************************************
//
//   PLACAR ELETRNICO
//
//   UTILIZA PIC 16F628A E 4 DISPLAY DE 7 SEGUIMENTOS
//
//
//   AUTOR : CLAUDIO LARIOS
//
//   data: 09/01/2018
//
//   Estilo:   TIME A  X   TIME B
//   Com 4 interruptores para ajuste:
//   reset, incrementa, decrementa e seleo
//
//   Uso didtico
//
//   Blog PICSOURCE (LARIOS.TECNOLOGIA.WS)
//
//******************************************************************************

  #DEFINE CATODO_COMUM //descomente p/ catodo comum e comente p/ anodo comum
//==============================================================================
#include <16F628a.h>
#fuses INTRC_IO,NOWDT,NOLVP,NOMCLR
#use delay(clock=4000000)    //tempo por instruo=1us
#rom 0x2100 = {1,2,3,4}
//bytes
#byte  porta   = 0x05
#byte  portb   = 0x06
#byte  trisa   = 0x85
#byte  trisb   = 0x86
#bit  gie      = 0x0b.7
#bit  peie     = 0x0b.6
#bit  t0ie     = 0x0b.5
//bits
#bit   zerar   = 0x05.5  //interruptor 'zerar'

#bit   sw_up   = 0x05.6  //interruptor 'up'
#bit   sw_down = 0x05.7  //interruptor 'down'
#bit   power   = 0x05.4  //sensor de falta de energia
#bit   sw_sel  = 0x06.7  //interruptor 'seleo'

//variavis globais
unsigned int8  dig3,dig2,dig1,dig0,mux;
unsigned int1 flag_sel=0;
// Tabela para retorno dos seguimentos correspondentes aos nmeros 0-9
byte const tabela [] = { 0xc0,0xf9,0xa4,0xb0,0x99,0x92,0x82,0xf8,0x80,0x90};


//==============================================================================
//          ROTINA DE INTERRUPO DO  TIMER 0
//==============================================================================
#INT_TIMER0
//multiplexa display de 7 seg
 void display_refress() {
  if(++mux>3){ mux=0;}
  porta=0; //apaga seguimentos para mudana
  switch (mux) {
  case 0:
   portb&=0x80;
   portb=portb|(0X7f & tabela[dig0]);
   #IFDEF  CATODO_COMUM
   portb^=0x7f;
   #ENDIF
   porta=1;
   break;
  case 1:
   portb&=0x80;
   portb=portb|(0X7f & tabela[dig1]);
   #IFDEF  CATODO_COMUM
   portb^=0x7f;
   #ENDIF
   porta=2;
   break;
  case 2:
   portb&=0x80;
   portb=portb|(0X7f & tabela[dig2]);
   #IFDEF  CATODO_COMUM
   portb^=0x7f;
   #ENDIF
   porta=4;
   break;
  case 3:
   portb&=0x80;
   portb=portb|(0X7f & tabela[dig3]);
   #IFDEF  CATODO_COMUM
   portb^=0x7f;
   #ENDIF
   porta=8;
   break;
  }
}
//==============================================================================
// RESETAR CONTAGEM (ZERA  SEM APAGAR TODOS DISPLAY COMO OCORRE EM UM 'RESET')
//==============================================================================
void  zera(){
       if(!zerar) {
          dig0=0;  //zera contadores inicialmente
          dig1=0;
          dig2=0;
          dig3=0;
          while(!zerar) delay_ms(100);//aguarda soltar boto
       }
}
//==============================================================================
//           ROTINA DE INCREMENTO DA CONTAGEM
//==============================================================================
void incrementa() {
   if(flag_sel){
     if((dig0==9) && (dig1==9))return;//limita a '99'
     if(++dig0>9) {
       dig0=0;
       if(++dig1>9) {
       dig1=0;
      }
    }
   }
   else{
     if((dig2==9) && (dig3==9))return;//limita a '99'
     if(++dig2>9){
       dig2=0;
       if(++dig3>9){
       dig3=0;
       }
     }
  }
}
//==============================================================================
//           ROTINA DE DECREMENTO DA CONTAGEM
//==============================================================================
void decrementa() {

  if(flag_sel){
     if(!dig0 && !dig1)return; //limita em '00'
     if(--dig0==0xff) {
      dig0=9;
      if(--dig1==0xff) {
      dig1=9;
      }
     }
  }
  else{
      if(!dig2 && !dig3)return; //limita em '00'
      if(--dig2==0xff){
        dig2=9;
        if(--dig3==0xff){
        dig3=9;
        }
     }
   }
}

//==============================================================================
//           ROTINA PRINCIPAL
//==============================================================================

void main() {
   setup_comparator(NC_NC_NC_NC);
   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_8);
   setup_timer_1(T1_DISABLED);
   #IFDEF CATODO_COMUM
   portb&=0x80;
   #ELSE
   portb|=0X7F;
   #ENDIF
   porta=0;
   trisb= 0b10000000;  //sentido das portas
   trisa= 0b11110000;
   delay_ms(500);
   dig0=read_eeprom(0);//recupera ultima contagem
   if(dig0>9)dig0=0;
   dig1=read_eeprom(1);
   if(dig1>9)dig1=0;
   dig2=read_eeprom(2);
   if(dig2>9)dig2=0;
   dig3=read_eeprom(3);
   if(dig3>9)dig3=0;

   flag_sel=0;
   t0ie=1;//habilita interrupes tmr0
   gie=1; //habilita interrupes geral

//==============================================================================
//           LOOP  PRINCIPAL
//==============================================================================


   for(;;) {

      if(sw_sel && sw_up && sw_down)flag_sel=0;
      zera();//testa boto de reset
      if(!sw_up) {incrementa();}//incrementa contagem

      while(!sw_up){
         delay_ms(50);//aguarda solta boto 'up
      }
      if(!sw_down){decrementa();}  //decrementa contagem

      while(!sw_down){
        delay_ms(50);//aguarda soltar boto 'down'
      }
      if(!sw_sel && !flag_sel)flag_sel=1;
      delay_ms(100);
      if(read_eeprom(0)!=dig0)write_eeprom(0,dig0); //salva se diferente
      if(read_eeprom(1)!=dig1)write_eeprom(1,dig1);
      if(read_eeprom(2)!=dig2)write_eeprom(2,dig2);
      if(read_eeprom(3)!=dig3)write_eeprom(3,dig3);

  }//while
}//main
